/************************************************************************/
/*									*/
/*			AD Conversion Example				*/
/*									*/
/*		Author: P. Dannegger					*/
/*			danni@specs.de					*/
/*									*/
/************************************************************************/
#include "adc.h"

#define MAX_VAL		20.00		// 20.00V
#define DECIMALS	.01		// .01

#define	DP3	0
#define DP2	_0DP			// set decimal point
#define DP1	0

#define SCALE ((u32)((65536.0 * MAX_VAL / DECIMALS + 512) / 1024))

//------------------------------------------------------------------------
//				T1 Interrupt
//------------------------------------------------------------------------

SIGNAL( SIG_OVERFLOW1 )
{
  PORTB = 1<<unused0^1<<unused1;	//digits off for ADC noise reduction !
  ADCSR = 1<<ADEN^1<<ADSC^1<<ADIF^1<<ADIE^1<<ADPS1^1<<ADPS0;
					//1MHz/8 = 125kHz = 104s conversion
}

//------------------------------------------------------------------------
//				ADC Interrupt
//------------------------------------------------------------------------

SIGNAL( SIG_ADC )
{
  static u32 adc_sum;
  static u8 count;
  u8 i;

  adc_sum += ADC;

  count++;

  if( count == 0 ){			// 4 values/s to be ergonomic !
    adc_val = adc_sum >> 8;		// average over 256 samples
    adc_sum = 0;
  }

  i = count & 3;			// 0, 1, 2, 3, 0 ...

  if( i == 0 ){				// output Digits
    PORTA = digit[0];			// Digit pattern 0
    PORTB |= 1<<pin_d0;			// Digit 0 on (common anode)
    return;
  }
  if( i == 1 ){
    PORTA = digit[1];
    PORTB |= 1<<pin_d1;
    return;
  }
  if( i == 2 ){
    PORTA = digit[2];
    PORTB |= 1<<pin_d2;
    return;
  }
  PORTA = digit[3];
  PORTB |= 1<<pin_d3;
}

//-------------------------------------------------------------------------
//				Initialization
//-------------------------------------------------------------------------

void init( void )
{
  DDRB = 1<<pin_d0^1<<pin_d1^1<<pin_d2^1<<pin_d3;	// enable outputs
  PORTB = 1<<unused0^1<<unused1;	// unused = pull up
  DDRA = 0xFF;				// digits output
  TCCR1B = 1<<CS11^1<<CS10;		//T1: prescaler 1/4
  TIMSK =  1<<TOIE1;			//enable T1 overflow int
  ADMUX = 1<<REFS1^1<<MUX3;		//UREF = 2.56V, select ADC8
}

//-------------------------------------------------------------------------
//			Decimal output 4 digits 0000-9999
//-------------------------------------------------------------------------

void dec_out( u16 val )
{
  u8 *ptr7s;				// point to 7-segment table

  while( val >= 10000 ){
    val -= 10000;
  }

  ptr7s = numbers1;
  while( val >= 1000 ){
    val -= 1000;
    ptr7s++;
  }
  digit[3] = *ptr7s ^ DP3;

  ptr7s = numbers0;
  while( val >= 100 ){
    val -= 100;
    ptr7s++;
  }
  digit[2] = *ptr7s ^ DP2;

  ptr7s = numbers1;
  while( val >= 10 ){
    val -= 10;
    ptr7s++;
  }
  digit[1] = *ptr7s ^ DP1;

  digit[0] = numbers0[val];
}


int main ( void )
{
  init();
  sei();

  for(;;){
    dec_out( (adc_val * SCALE) >> 16 );
  }
}
